@@ -35,29 +35,37 @@ prepanel.default.densityplot <-
 {
     if (!is.numeric(x)) x <- as.numeric(x)
 
-    if (length(x)<1)
+    if (length(x) < 1)
         list(xlim = NA,
              ylim = NA,
              dx = NA,
              dy = NA)
     else if (is.null(groups))
     {
-        h <- do.call("density", c(list(x=x), darg))
-        list(xlim = range(h$x),
-             ylim = range(h$y),
-             dx = diff(h$x), dy = diff(h$y))
+        if (length(x) > 1)
+        {
+            h <- do.call("density", c(list(x=x), darg))
+            list(xlim = range(h$x),
+                 ylim = range(h$y),
+                 dx = diff(h$x), dy = diff(h$y))
+        }
+        else
+            list(xlim = range(x),
+                 ylim = 0,
+                 dx = 1, dy = 1)
     }
     else
     {
         vals <- sort(unique(groups))
         nvals <- length(vals)
-        xl <- numeric(0)
-        yl <- numeric(0)
+        xl <- range(x)
+        yl <- 0
         dxl <- numeric(0) # bad names !!
         dyl <- numeric(0) 
         for (i in seq(along=vals)) {
             id <- (groups[subscripts] == vals[i])
-            if (any(id)) {
+            if (sum(id) > 1)
+            {
                 h <- do.call("density", c(list(x=x[id]), darg))
                 xl <- c(xl, h$x)
                 yl <- c(yl, h$y)
@@ -77,29 +85,30 @@ panel.densityplot <-
              darg = list(n = 30),
              plot.points = TRUE,
              ref = FALSE,
-             cex = 0.5,
              col = plot.line$col,
-             col.line,
+             col.line = col,
              ...)
 {
     x <- as.numeric(x)
 
-    if (ref) {
+    if (ref)
+    {
         reference.line <- trellis.par.get("reference.line")
         panel.abline(h=0,
                      col = reference.line$col,
                      lty = reference.line$lty,
                      lwd = reference.line$lwd)
     }
-    if (length(x)>1) {
-        plot.line <- trellis.par.get("plot.line")
-        if (missing(col.line)) col.line <- col
+
+    plot.line <- trellis.par.get("plot.line")
+    if (length(x) > 1)
+    {
         h <- do.call("density", c(list(x=x), darg))
         lim <- current.viewport()$xscale
         id <- (h$x>=lim[1] & h$x<=lim[2])
         llines(x = h$x[id], y = h$y[id], col = col.line, ...)
-        if (plot.points) panel.xyplot(x = x, y = rep(0, length(x)), cex = cex, col = col, ...) 
     }
+    if (plot.points) panel.xyplot(x = x, y = rep(0, length(x)), col = col, ...) 
 }
 
 
@@ -109,11 +118,10 @@ panel.densityplot <-
 densityplot <-
     function(formula,
              data = parent.frame(),
-             allow.multiple = FALSE,
+             allow.multiple = is.null(groups) || outer,
              outer = FALSE,
              auto.key = FALSE,
              aspect = "fill",
-             layout = NULL,
              panel = if (is.null(groups)) "panel.densityplot" else "panel.superpose",
              prepanel = NULL,
              scales = list(),
@@ -134,6 +142,7 @@ densityplot <-
              to = NULL,
              cut = NULL,
              na.rm = NULL,
+             drop.unused.levels = TRUE,
              ...,
              panel.groups = "panel.densityplot",
              subscripts = !is.null(groups),
@@ -171,7 +180,8 @@ densityplot <-
     form <-
         latticeParseFormula(formula, data, subset = subset,
                             groups = groups, multiple = allow.multiple,
-                            outer = outer, subscripts = TRUE)
+                            outer = outer, subscripts = TRUE,
+                            drop = drop.unused.levels)
 
     groups <- form$groups
 
@@ -192,7 +202,6 @@ densityplot <-
     if (number.of.cond == 0) {
         strip <- FALSE
         cond <- list(as.factor(rep(1, length(x))))
-        layout <- c(1,1,1)
         number.of.cond <- 1
     }
 
@@ -206,23 +215,18 @@ densityplot <-
     ## create a skeleton trellis object with the
     ## less complicated components:
     foo <- do.call("trellis.skeleton",
-                   c(list(aspect = aspect,
+                   c(list(cond = cond,
+                          aspect = aspect,
                           strip = strip,
                           panel = panel,
                           xlab = xlab,
-                          ylab = ylab), dots))
+                          ylab = ylab,
+                          xlab.default = form$right.name,
+                          ylab.default = "Density"), dots))
 
     dots <- foo$dots # arguments not processed by trellis.skeleton
     foo <- foo$foo
     foo$call <- match.call()
-    foo$fontsize.normal <- 10
-    foo$fontsize.small <- 8
-
-    ## This is for cases like xlab/ylab = list(cex=2)
-    if (is.list(foo$xlab) && !is.characterOrExpression(foo$xlab$label))
-        foo$xlab$label <- form$right.name
-    if (is.list(foo$ylab) && !is.characterOrExpression(foo$ylab$label))
-        foo$ylab$label <- "Density"
 
     ## Step 2: Compute scales.common (leaving out limits for now)
 
@@ -276,8 +280,6 @@ densityplot <-
     if (!any(!id.na)) stop("nothing to draw")
     ## Nothing simpler ?
 
-    foo$condlevels <- lapply(cond, levels)
-
     ## Step 6: Evaluate layout, panel.args.common and panel.args
 
     foo$panel.args.common <- c(dots, list(darg = darg))
@@ -286,49 +288,42 @@ densityplot <-
         foo$panel.args.common$panel.groups <- panel.groups
     }
 
-    layout <- compute.layout(layout, cond.max.level, skip = foo$skip)
-    plots.per.page <- max(layout[1] * layout[2], layout[2])
-    number.of.pages <- layout[3]
-    foo$skip <- rep(foo$skip, length = plots.per.page)
-    foo$layout <- layout
-    nplots <- plots.per.page * number.of.pages
 
-    foo$panel.args <- as.list(1:nplots)
-    cond.current.level <- rep(1,number.of.cond)
-    panel.number <- 1 # this is a counter for panel number
-    for (page.number in 1:number.of.pages)
-        if (!any(cond.max.level-cond.current.level<0))
-            for (plot in 1:plots.per.page) {
+    nplots <- prod(cond.max.level)
+    if (nplots != prod(sapply(foo$condlevels, length))) stop("mismatch")
+    foo$panel.args <- vector(mode = "list", length = nplots)
 
-                if (foo$skip[plot]) foo$panel.args[[panel.number]] <- FALSE
-                else if(!any(cond.max.level-cond.current.level<0)) {
 
-                    id <- !id.na
-                    for(i in 1:number.of.cond)
-                    {
-                        var <- cond[[i]]
-                        id <- id &
-                        if (is.shingle(var))
-                            ((var >=
-                              levels(var)[[cond.current.level[i]]][1])
-                             & (var <=
-                                levels(var)[[cond.current.level[i]]][2]))
-                        else (as.numeric(var) == cond.current.level[i])
-                    }
+    cond.current.level <- rep(1, number.of.cond)
 
-                    foo$panel.args[[panel.number]] <-
-                        list(x = x[id])
-                    if (subscripts)
-                        foo$panel.args[[panel.number]]$subscripts <-
-                            subscr[id]
 
-                    cond.current.level <-
-                        cupdate(cond.current.level,
-                                cond.max.level)
-                }
+    for (panel.number in seq(length = nplots))
+    {
 
-                panel.number <- panel.number + 1
-            }
+        
+        id <- !id.na
+        for(i in 1:number.of.cond)
+        {
+            var <- cond[[i]]
+            id <- id &
+            if (is.shingle(var))
+                ((var >=
+                  levels(var)[[cond.current.level[i]]][1])
+                 & (var <=
+                    levels(var)[[cond.current.level[i]]][2]))
+            else (as.numeric(var) == cond.current.level[i])
+        }
+
+        foo$panel.args[[panel.number]] <-
+            list(x = x[id])
+        if (subscripts)
+            foo$panel.args[[panel.number]]$subscripts <-
+                subscr[id]
+
+        cond.current.level <-
+            cupdate(cond.current.level,
+                    cond.max.level)
+    }
 
     foo <- c(foo,
              limits.and.aspect(prepanel.default.densityplot,
@@ -345,11 +340,26 @@ densityplot <-
                                y.axs = foo$y.scales$axs))
 
 
-    if (is.null(foo$key) && !is.null(groups) &&
+    if (is.null(foo$legend) && !is.null(groups) &&
         (is.list(auto.key) || (is.logical(auto.key) && auto.key)))
-        foo$key <- do.call("simpleKey",
-                           c(list(levels(as.factor(groups))),
-                             if (is.list(auto.key)) auto.key else list()))
+    {
+        foo$legend <-
+            list(list(fun = "drawSimpleKey",
+                      args =
+                      c(list(levels(as.factor(groups))),
+                        if (is.list(auto.key)) auto.key else list())))
+        foo$legend[[1]]$x <- foo$legend[[1]]$args$x
+        foo$legend[[1]]$y <- foo$legend[[1]]$args$y
+        foo$legend[[1]]$corner <- foo$legend[[1]]$args$corner
+
+        names(foo$legend) <- 
+            if (any(c("x", "y", "corner") %in% names(foo$legend[[1]]$args)))
+                "inside"
+            else
+                "top"
+        if (!is.null(foo$legend[[1]]$args$space))
+            names(foo$legend) <- foo$legend[[1]]$args$space
+    }
 
     class(foo) <- "trellis"
     foo
